import { Router } from "express";
import Register from "./register";
import Login from "./login";
import TwoFactorRoute from "./twofactor";
import { GetToken, DeleteToken } from "./token";
import { GetAccount } from "./account";

const UserRoute: Router = Router();

/**
 * @api {post} /user/register
 * @apiName UserRegister
 * 
 * @apiGroup user
 * @apiPermission none
 * 
 * @apiParam  {String} mail EMail linked to this Account
 * @apiParam  {String} username The new Username
 * @apiParam  {String} password Password hashed and salted like specification
 * @apiParam  {String} salt The Salt used for password hashing
 * @apiParam  {String} regcode The regcode, that should be used
 * @apiParam  {String} gender Gender can be: "male", "female", "other", "none"
 * @apiParam  {String} name The real name of the User
 * 
 * @apiSuccess {Boolean} success
 * 
 * @apiErrorExample {Object} Error-Response:
     {
        error: [
            {
                message: "Some Error",
                field: "username"
            }
        ],
        status: 400
     }
 */
UserRoute.post("/register", Register);

/**
 * @api {post} /user/login?type=:type
 * @apiName UserLogin
 * 
 * @apiParam {String} type Type could be either "username" or "password"
 * 
 * @apiGroup user
 * @apiPermission none
 * 
 * @apiParam  {String} username Username (either username or uid required)
 * @apiParam  {String} uid (either username or uid required)
 * @apiParam  {String} password Password hashed and salted like specification (only on type password)
 * 
 * @apiSuccess {String} uid On type = "username"
 * @apiSuccess {String} salt On type = "username"
 * 
 * @apiSuccess {String} login On type = "password". Login Token
 * @apiSuccess {String} special On type = "password". Special Token
 * @apiSuccess {Object[]} tfa Will be set when TwoFactorAuthentication is required
 * @apiSuccess {String} tfa.id The ID of the TFA Method
 * @apiSuccess {String} tfa.name The name of the TFA Method
 * @apiSuccess {String} tfa.type The type of the TFA Method
 */
UserRoute.post("/login", Login)
UserRoute.use("/twofactor", TwoFactorRoute);

/**
 * @api {get} /user/token
 * @apiName UserGetToken
 * 
 * @apiGroup user
 * @apiPermission user
 * 
 * @apiSuccess {Object[]} token
 * @apiSuccess {String} token.id The Token ID
 * @apiSuccess {String} token.special Identifies Special Token
 * @apiSuccess {String} token.ip IP the token was optained from
 * @apiSuccess {String} token.browser The Browser the token was optained from (User Agent)
 * @apiSuccess {Boolean} token.isthis Shows if it is token used by this session
 */
UserRoute.get("/token", GetToken);

/**
 * @api {delete} /user/token/:id
 * @apiParam {String} id The id of the token to be deleted
 * 
 * @apiName UserDeleteToken
 * 
 * 
 * @apiGroup user
 * @apiPermission user
 * 
 * @apiSuccess {Boolean} success
 */
UserRoute.delete("/token/:id", DeleteToken);


/**
 * @api {delete} /user/account
 * @apiName UserGetAccount
 * 
 * @apiGroup user
 * @apiPermission user
 * 
 * @apiSuccess {Boolean} success
 * @apiSuccess {Object[]} user
 * @apiSuccess {String} user.id User ID
 * @apiSuccess {String} token.name Full name of the user
 * @apiSuccess {String} token.username Username of user
 * @apiSuccess {Date} token.birthday Birthday
 * @apiSuccess {Number} token.gender Gender of user (none = 0, male = 1, female = 2, other = 3)
 */
UserRoute.get("/account", GetAccount);
export default UserRoute;